@@ -81,10 +81,6 @@ struct GdmFactorySlavePrivate
         DBusGConnection   *connection;
 };
 
-enum {
-        PROP_0,
-};
-
 static void     gdm_factory_slave_class_init    (GdmFactorySlaveClass *klass);
 static void     gdm_factory_slave_init          (GdmFactorySlave      *factory_slave);
 static void     gdm_factory_slave_finalize      (GObject             *object);
@@ -868,55 +864,12 @@ gdm_factory_slave_stop (GdmSlave *slave)
         return TRUE;
 }
 
-static void
-gdm_factory_slave_set_property (GObject      *object,
-                               guint          prop_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
-{
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static void
-gdm_factory_slave_get_property (GObject    *object,
-                                guint       prop_id,
-                                GValue     *value,
-                                GParamSpec *pspec)
-{
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static GObject *
-gdm_factory_slave_constructor (GType                  type,
-                               guint                  n_construct_properties,
-                               GObjectConstructParam *construct_properties)
-{
-        GdmFactorySlave      *factory_slave;
-
-        factory_slave = GDM_FACTORY_SLAVE (G_OBJECT_CLASS (gdm_factory_slave_parent_class)->constructor (type,
-                                                                                                         n_construct_properties,
-                                                                                                         construct_properties));
-
-        return G_OBJECT (factory_slave);
-}
-
 static void
 gdm_factory_slave_class_init (GdmFactorySlaveClass *klass)
 {
         GObjectClass  *object_class = G_OBJECT_CLASS (klass);
         GdmSlaveClass *slave_class = GDM_SLAVE_CLASS (klass);
 
-        object_class->get_property = gdm_factory_slave_get_property;
-        object_class->set_property = gdm_factory_slave_set_property;
-        object_class->constructor = gdm_factory_slave_constructor;
         object_class->finalize = gdm_factory_slave_finalize;
 
         slave_class->start = gdm_factory_slave_start;
